#include "Scene.hxx"
#include "PerspectiveCamera.hxx"

Scene::Scene()
  : m_camera(new PerspectiveCamera(Vec3f(0,0,8), 
				   Vec3f(0,0,-1), 
				   Vec3f(0,1,0), 
				   50, 
				   640, 
				   480)
	     ),
    m_bgColor(Vec3f(0,0,0))
{
}

Scene::~Scene()
{
  delete m_camera;
}

Scene::Scene(const Scene& s)
{
  operator=(s);
}

Scene&
Scene::operator=(const Scene& s)
{
  return *this;
}

void 
Scene::Add(Primitive* p)
{
  m_primitives.push_back(p);
}

void
Scene::Add(Light* l)
{
}

bool
Scene::Intersect(Ray& ray)
{
  bool intersect = false;

  for(unsigned int i = 0; i < m_primitives.size(); ++i)
    {
      intersect |= m_primitives[i]->Intersect(ray);
    }
  
  return intersect;
}

bool
Scene::Occluded(Ray& ray)
{
  return false;
}

Vec3f
Scene::RayTrace(Ray& ray)
{
  bool intersect = this->Intersect(ray);
  return (intersect) ? ray.hit()->shader()->Shade(ray) : m_bgColor;
}

const Camera* 
Scene::camera() const
{
  return m_camera;
}
